package cn.com.qmth.examcloud.core.basic.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.security.enums.RoleMeta;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserDataRuleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserDataRuleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.service.UserDataRuleService;
import cn.com.qmth.examcloud.core.basic.service.UserService;
import cn.com.qmth.examcloud.core.basic.service.bean.datarule.*;
import cn.com.qmth.examcloud.core.basic.service.cache.UserDataRuleCacheBean;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamMapsReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamMapsResp;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class UserDataRuleServiceImpl implements UserDataRuleService {

    private static final Logger LOG = LoggerFactory.getLogger(UserDataRuleServiceImpl.class);

    private static final int DATA_RULE_CACHE_TIMEOUT = 600;// 过期时间（秒）

    private static final long DEFAULT_ID = -1L;// 为-1时，代表“全部”

    @Autowired
    private UserDataRuleRepo userDataRuleRepo;

    @Autowired
    private UserRepo userRepo;

    @Autowired
    private OrgRepo orgRepo;

    @Autowired
    private CourseRepo courseRepo;

    @Autowired
    private UserService userService;

    @Autowired
    private ExamCloudService examCloudService;

    @Autowired
    private RedisClient redisClient;

    @Override
    public boolean getUserDataRuleForDefaultStatus(Long userId, DataRuleType type) {
        if (userId == null) {
            throw new StatusException("400400", "用户ID不能为空！");
        }

        if (type == null) {
            throw new StatusException("400400", "权限类型不能为空！");
        }

        UserDataRuleEntity userDataRule = userDataRuleRepo.findByUserIdAndTypeAndRefId(userId, type, DEFAULT_ID);
        if (userDataRule != null) {
            return userDataRule.getEnabled();
        }

        // 不存在时，默认false
        return false;
    }

    @Override
    public void updateUserDataRuleForDefaultStatus(Long userId, DataRuleType type, boolean enabled) {
        if (userId == null) {
            throw new StatusException("400400", "用户ID不能为空！");
        }

        if (type == null) {
            throw new StatusException("400400", "权限类型不能为空！");
        }

        UserDataRuleEntity entity = userDataRuleRepo.findByUserIdAndTypeAndRefId(userId, type, DEFAULT_ID);
        if (entity == null) {
            Optional<UserEntity> user = userRepo.findById(userId);
            if (!user.isPresent()) {
                throw new StatusException("400400", "此用户不存在！");
            }

            entity = new UserDataRuleEntity();
            entity.setRootOrgId(user.get().getRootOrgId());
            entity.setUserId(userId);
            entity.setType(type);
            entity.setRefId(DEFAULT_ID);
            entity.setEnabled(enabled);
        } else {
            entity.setEnabled(enabled);
        }

        userDataRuleRepo.save(entity);

        // 清理缓存
        this.clearUserDataRuleCache(userId, type);
    }

    @Override
    public Page<UserDataRuleInfo> getUserDataRuleList(UserDataRuleQuery req) {
        if (req.getUserId() == null) {
            throw new StatusException("400400", "用户ID不能为空！");
        }

        Specification<UserDataRuleEntity> spec = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("userId"), req.getUserId()));

            if (req.getType() != null) {
                predicates.add(cb.equal(root.get("type"), req.getType()));
            }

            if (req.getRefId() != null) {
                predicates.add(cb.equal(root.get("refId"), req.getRefId()));
            } else {
                predicates.add(cb.notEqual(root.get("refId"), DEFAULT_ID));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Sort sort = Sort.by(Sort.Order.desc("id"));
        Pageable pageable = PageRequest.of(req.getPageNo() - 1, req.getPageSize(), sort);
        Page<UserDataRuleEntity> page = userDataRuleRepo.findAll(spec, pageable);

        if (!page.hasContent()) {
            return new PageImpl<>(new ArrayList<>(), pageable, 0);
        }

        List<UserDataRuleInfo> list = page.getContent().stream().map(entity -> {
            UserDataRuleInfo info = new UserDataRuleInfo();
            info.setId(entity.getId());
            info.setUserId(entity.getUserId());
            info.setType(entity.getType());
            info.setRefId(entity.getRefId());
            info.setCreationTime(entity.getCreationTime());
            info.setUpdateTime(entity.getUpdateTime());
            return info;
        }).collect(Collectors.toList());

        return new PageImpl<>(list, pageable, page.getTotalElements());
    }

    @Override
    public void addUserDataRules(UserDataRuleForm req) {
        if (req.getUserId() == null) {
            throw new StatusException("400400", "用户ID不能为空！");
        }

        if (req.getType() == null) {
            throw new StatusException("400400", "权限类型不能为空！");
        }

        if (CollectionUtils.isEmpty(req.getRefIds())) {
            throw new StatusException("400400", "关联ID不能为空！");
        }

        Optional<UserEntity> user = userRepo.findById(req.getUserId());
        if (!user.isPresent()) {
            throw new StatusException("400400", "此用户不存在！");
        }

        for (Long refId : req.getRefIds()) {
            boolean existed = userDataRuleRepo.existsByUserIdAndTypeAndRefId(req.getUserId(), req.getType(), refId);
            if (existed) {
                continue;
            }

            UserDataRuleEntity entity = new UserDataRuleEntity();
            entity.setRootOrgId(user.get().getRootOrgId());
            entity.setUserId(req.getUserId());
            entity.setType(req.getType());
            entity.setRefId(refId);
            entity.setEnabled(true);
            userDataRuleRepo.save(entity);
        }

        // 清理缓存
        this.clearUserDataRuleCache(req.getUserId(), req.getType());
    }

    @Override
    public void addUserDataRule(Long userId, DataRuleType type, Long refId) {
        if (refId == null) {
            throw new StatusException("400400", "关联ID不能为空！");
        }

        UserDataRuleForm req = new UserDataRuleForm();
        req.setUserId(userId);
        req.setType(type);
        req.addRefId(refId);

        this.addUserDataRules(req);
    }

    @Override
    public void deleteUserDataRule(UserDataRuleForm req) {
        if (req.getUserId() == null) {
            throw new StatusException("400400", "用户ID不能为空！");
        }

        if (req.getType() == null) {
            throw new StatusException("400400", "权限类型不能为空！");
        }

        if (CollectionUtils.isEmpty(req.getRefIds())) {
            throw new StatusException("400400", "关联ID不能为空！");
        }

        userDataRuleRepo.deleteByUserIdAndTypeAndRefIds(req.getUserId(), req.getType(), req.getRefIds());

        // 清理缓存
        this.clearUserDataRuleCache(req.getUserId(), req.getType());
    }

    @Override
    public void copyUserDataRule(UserDataRuleCopyReq req) {
        if (req.getSourceUserId() == null) {
            throw new StatusException("400400", "源用户ID不能为空！");
        }

        if (CollectionUtils.isEmpty(req.getTargetUserIds())) {
            throw new StatusException("400400", "目标用户ID不能为空！");
        }

        Optional<UserEntity> sourceUser = userRepo.findById(req.getSourceUserId());
        if (!sourceUser.isPresent()) {
            throw new StatusException("400400", "源用户不存在！");
        }

        Set<String> sourceRoleCodes = userService.getUserRoleCodes(req.getSourceUserId());
        if (sourceRoleCodes.contains(RoleMeta.SUPER_ADMIN.name()) || sourceRoleCodes.contains(RoleMeta.ORG_ADMIN.name())) {
            throw new StatusException("400400", "超管或机构管理员的数据权限不可复制！");
        }

        Map<Long, Long> targetUserMaps = new HashMap<>();
        for (Long targetUserId : req.getTargetUserIds()) {
            if (targetUserId.equals(req.getSourceUserId())) {
                LOG.warn(String.format("跳过复制，targetUserId与sourceUserId相同... id = %s", targetUserId));
                continue;
            }

            Optional<UserEntity> targetUser = userRepo.findById(targetUserId);
            if (!targetUser.isPresent()) {
                LOG.warn(String.format("User is not exist... targetUserId = %s", targetUserId));
                continue;
            }

            Set<String> targetRoleCodes = userService.getUserRoleCodes(targetUserId);
            if (targetRoleCodes.contains(RoleMeta.SUPER_ADMIN.name()) || targetRoleCodes.contains(RoleMeta.ORG_ADMIN.name())) {
                throw new StatusException("400400", "超管、机构管理员的数据权限不可复制！");
            }

            targetUserMaps.put(targetUserId, targetUser.get().getRootOrgId());
        }

        if (targetUserMaps.isEmpty()) {
            return;
        }

        List<UserDataRuleEntity> sourceUserRules = userDataRuleRepo.findByUserId(req.getSourceUserId());

        for (Map.Entry<Long, Long> targetUser : targetUserMaps.entrySet()) {
            userDataRuleRepo.deleteByUserId(targetUser.getKey());

            for (UserDataRuleEntity sourceUserRule : sourceUserRules) {
                UserDataRuleEntity copyRule = new UserDataRuleEntity();
                copyRule.setRootOrgId(targetUser.getValue());
                copyRule.setUserId(targetUser.getKey());
                copyRule.setType(sourceUserRule.getType());
                copyRule.setRefId(sourceUserRule.getRefId());
                copyRule.setEnabled(sourceUserRule.getEnabled());
                userDataRuleRepo.save(copyRule);
            }

            for (DataRuleType type : DataRuleType.values()) {
                // 清理缓存
                this.clearUserDataRuleCache(targetUser.getKey(), type);
            }
        }
    }

    @Override
    public Page<UserDataRuleForOrgInfo> getUserDataRuleListForOrg(UserDataRuleQuery req) {
        req.setType(DataRuleType.ORG);

        Page<UserDataRuleInfo> page = this.getUserDataRuleList(req);
        if (!page.hasContent()) {
            return new PageImpl<>(new ArrayList<>(), page.getPageable(), page.getTotalElements());
        }

        Set<Long> orgIds = page.getContent().stream().map(UserDataRuleInfo::getRefId).collect(Collectors.toSet());
        List<OrgEntity> orgList = orgRepo.findByIdIn(orgIds);
        Map<Long, OrgEntity> orgMaps = orgList.stream().collect(Collectors.toMap(OrgEntity::getId, v -> v, (k, v) -> v));

        List<UserDataRuleForOrgInfo> list = new ArrayList<>();
        for (UserDataRuleInfo info : page.getContent()) {
            UserDataRuleForOrgInfo allInfo = new UserDataRuleForOrgInfo(info);

            OrgEntity org = orgMaps.get(info.getRefId());
            if (org != null) {
                allInfo.setOrgCode(org.getCode());
                allInfo.setOrgName(org.getName());
                allInfo.setContacts(org.getContacts());
            }

            list.add(allInfo);
        }

        return new PageImpl<>(list, page.getPageable(), page.getTotalElements());
    }

    @Override
    public Page<UserDataRuleForExamInfo> getUserDataRuleListForExam(UserDataRuleQuery req) {
        req.setType(DataRuleType.EXAM);

        Page<UserDataRuleInfo> page = this.getUserDataRuleList(req);
        if (!page.hasContent()) {
            return new PageImpl<>(new ArrayList<>(), page.getPageable(), page.getTotalElements());
        }

        Set<Long> examIds = page.getContent().stream().map(UserDataRuleInfo::getRefId).collect(Collectors.toSet());
        GetExamMapsReq examsReq = new GetExamMapsReq();
        examsReq.setExamIds(examIds);
        GetExamMapsResp examsResp = examCloudService.getExamMaps(examsReq);
        Map<Long, ExamBean> examMaps = examsResp.getExamMaps();

        List<UserDataRuleForExamInfo> list = new ArrayList<>();
        for (UserDataRuleInfo info : page.getContent()) {
            UserDataRuleForExamInfo allInfo = new UserDataRuleForExamInfo(info);

            ExamBean exam = examMaps.get(info.getRefId());
            if (exam != null) {
                allInfo.setExamCode(exam.getCode());
                allInfo.setExamName(exam.getName());
                allInfo.setExamType(exam.getExamType());
            }

            list.add(allInfo);
        }

        return new PageImpl<>(list, page.getPageable(), page.getTotalElements());
    }

    @Override
    public Page<UserDataRuleForCourseInfo> getUserDataRuleListForCourse(UserDataRuleQuery req) {
        req.setType(DataRuleType.COURSE);

        Page<UserDataRuleInfo> page = this.getUserDataRuleList(req);
        if (!page.hasContent()) {
            return new PageImpl<>(new ArrayList<>(), page.getPageable(), page.getTotalElements());
        }

        Set<Long> courseIds = page.getContent().stream().map(UserDataRuleInfo::getRefId).collect(Collectors.toSet());
        List<CourseEntity> courseList = courseRepo.findByIdIn(courseIds);
        Map<Long, CourseEntity> courseMaps = courseList.stream().collect(Collectors.toMap(CourseEntity::getId, v -> v, (k, v) -> v));

        List<UserDataRuleForCourseInfo> list = new ArrayList<>();
        for (UserDataRuleInfo info : page.getContent()) {
            UserDataRuleForCourseInfo allInfo = new UserDataRuleForCourseInfo(info);

            CourseEntity course = courseMaps.get(info.getRefId());
            if (course != null) {
                allInfo.setCourseCode(course.getCode());
                allInfo.setCourseName(course.getName());
                allInfo.setCourseLevel(course.getLevel().getName());
            }

            list.add(allInfo);
        }

        return new PageImpl<>(list, page.getPageable(), page.getTotalElements());
    }

    @Override
    public boolean checkUserDataRule(Long userId, DataRuleType type, Long refId) {
        if (refId == null) {
            throw new StatusException("400400", "关联ID不能为空！");
        }

        UserDataRuleCacheBean cacheBean = this.loadUserDataRuleCache(userId, type);

        if (cacheBean.getGlobalStatus()) {
            return cacheBean.getGlobalStatus();
        }

        return cacheBean.getRefIds().contains(refId);
    }

    @Override
    public UserDataRuleCacheBean loadUserDataRuleCache(Long userId, DataRuleType type) {
        if (userId == null) {
            throw new StatusException("400400", "用户ID不能为空！");
        }

        if (type == null) {
            throw new StatusException("400400", "权限类型不能为空！");
        }

        String cacheKey = CacheConstants.CACHE_B_DATA_RULE + userId + "_" + type.name();
        UserDataRuleCacheBean cacheBean = redisClient.get(cacheKey, UserDataRuleCacheBean.class);
        if (cacheBean != null) {
            return cacheBean;
        }

        Set<Long> refIds = new HashSet<>();
        cacheBean = new UserDataRuleCacheBean();
        cacheBean.setGlobalStatus(false);
        cacheBean.setRefIds(refIds);

        List<UserDataRuleEntity> rules = userDataRuleRepo.findByUserIdAndType(userId, type);
        for (UserDataRuleEntity rule : rules) {
            if (rule.getRefId().equals(DEFAULT_ID)) {
                cacheBean.setGlobalStatus(rule.getEnabled());
                continue;
            }
            refIds.add(rule.getRefId());
        }

        if (LOG.isInfoEnabled()) {
            LOG.info(String.format("userId = %s, dataRuleType = %s, globalStatus = %s, refIds = [%s]",
                    userId, type.name(), cacheBean.getGlobalStatus(), StringUtils.join(refIds, ",")));
        }

        redisClient.set(cacheKey, cacheBean, DATA_RULE_CACHE_TIMEOUT);
        return cacheBean;
    }

    private void clearUserDataRuleCache(Long userId, DataRuleType type) {
        String cacheKey = CacheConstants.CACHE_B_DATA_RULE + userId + "_" + type.name();
        redisClient.delete(cacheKey);
    }

}
