package cn.com.qmth.examcloud.core.basic.service.impl;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.UUID;
import cn.com.qmth.examcloud.core.basic.base.util.VerifyCode;
import cn.com.qmth.examcloud.core.basic.dao.enums.LoginRuleType;
import cn.com.qmth.examcloud.core.basic.service.AuthService;
import cn.com.qmth.examcloud.core.basic.service.LoginRuleService;
import cn.com.qmth.examcloud.core.basic.service.VerifyCodeService;
import cn.com.qmth.examcloud.core.basic.service.bean.GeetestLoginInfo;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginInfo;
import cn.com.qmth.examcloud.core.basic.service.bean.VerifyCodeLoginInfo;
import cn.com.qmth.examcloud.core.basic.service.cache.VerifyCodeCacheBean;
import cn.com.qmth.examcloud.reports.commons.bean.OnlineStudentReport;
import cn.com.qmth.examcloud.reports.commons.bean.OperateReport;
import cn.com.qmth.examcloud.reports.commons.enums.OperateContent;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 验证码相关接口
 */
@Service
public class VerifyCodeServiceImpl implements VerifyCodeService {

    private static final Logger LOG = LoggerFactory.getLogger(VerifyCodeServiceImpl.class);

    @Autowired
    private LoginRuleService loginRuleService;

    @Autowired
    private AuthService authService;

    @Autowired
    private RedisClient redisClient;

    /**
     * 极验-验证码登录接口
     *
     * @return 登录成功信息
     */
    @Override
    public User geetestLogin(GeetestLoginInfo info) {
        if (info.getRootOrgId() == null) {
            throw new StatusException("400", "顶级机构ID不能为空");
        }
        if (StringUtils.isBlank(info.getAccountType())) {
            throw new StatusException("400", "登陆账号类型不能为空");
        }
        if (StringUtils.isBlank(info.getAccountValue())) {
            throw new StatusException("400", "登陆账号不能为空");
        }
        if (StringUtils.isBlank(info.getPassword())) {
            throw new StatusException("400", "登陆密码不能为空");
        }

        // 是否在登录的白名单
        boolean allow = loginRuleService.isLoginRuleAllow(info.getRootOrgId(), LoginRuleType.STUDENT_CLIENT_LOGIN);
        if (!allow) {
            throw new StatusException("403002", "当前机构登录被禁用");
        }

        LoginInfo loginInfo = new LoginInfo();
        loginInfo.setRootOrgId(info.getRootOrgId());
        loginInfo.setAccountType(info.getAccountType());
        loginInfo.setAccountValue(info.getAccountValue());
        loginInfo.setPassword(info.getPassword());
        loginInfo.setClientIp(info.getIp_address());
        User user = authService.login(loginInfo, false);

        //操作日志
        ReportsUtil.report(new OperateReport(user.getRootOrgId(), user.getUserId(), user.getUserId(), null, UserType.STUDENT, OperateContent.STUDENT_LOGIN.getDesc()));

        // 在线学生登录打点
        ReportsUtil.report(new OnlineStudentReport(user.getRootOrgId(), user.getUserId()));

        return user;
    }

    /**
     * 数字公式-验证码生成接口
     *
     * @return 字符串：uuid(32位) + base64
     */
    @Override
    public String verifyCodeGenerate(Long rootOrgId, String accountValue) {
        final String cacheKeyVerifyCodeLimit = CacheConstants.CACHE_B_VERIFY_CODE_LIMIT + rootOrgId + "_" + accountValue;
        Boolean limit = redisClient.get(cacheKeyVerifyCodeLimit, Boolean.class);
        if (limit != null) {
            throw new StatusException("500", "操作太频繁，请稍后重试！");
        }
        redisClient.set(cacheKeyVerifyCodeLimit, true, 1);// N秒

        VerifyCode.Result result = VerifyCode.generateVerifyCode();
        byte[] bytes = VerifyCode.imageToBytes(result.getImage());
        String base64 = Base64.encodeBase64String(bytes);
        String uuid = UUID.randomUUID();

        LOG.info(String.format("accountValue = %s, uuid = %s, verifyCode = %s", accountValue, uuid, result.getResult()));
        VerifyCodeCacheBean cacheBean = new VerifyCodeCacheBean();
        cacheBean.setUuid(uuid);
        cacheBean.setVerifyCode(result.getResult());

        final String cacheKeyVerifyCode = CacheConstants.CACHE_B_VERIFY_CODE + rootOrgId + "_" + accountValue;
        redisClient.set(cacheKeyVerifyCode, cacheBean, 60);// N秒

        return uuid + base64;
    }

    /**
     * 数字公式-验证码登录接口
     *
     * @return 登录成功信息
     */
    @Override
    public User verifyCodeLogin(VerifyCodeLoginInfo info) {
        if (info.getRootOrgId() == null) {
            throw new StatusException("400", "顶级机构ID不能为空");
        }
        if (StringUtils.isBlank(info.getAccountType())) {
            throw new StatusException("400", "登陆账号类型不能为空");
        }
        if (StringUtils.isBlank(info.getAccountValue())) {
            throw new StatusException("400", "登陆账号不能为空");
        }
        if (StringUtils.isBlank(info.getPassword())) {
            throw new StatusException("400", "登陆密码不能为空");
        }
        if (info.getVerifyCode() == null) {
            throw new StatusException("400", "验证码不能为空");
        }

        // 是否在登录的白名单
        boolean allow = loginRuleService.isLoginRuleAllow(info.getRootOrgId(), LoginRuleType.STUDENT_CLIENT_LOGIN);
        if (!allow) {
            throw new StatusException("403002", "当前机构登录被禁用");
        }

        final String cacheKeyVerifyCode = CacheConstants.CACHE_B_VERIFY_CODE + info.getRootOrgId() + "_" + info.getAccountValue();
        VerifyCodeCacheBean cacheBean = redisClient.get(cacheKeyVerifyCode, VerifyCodeCacheBean.class);
        if (cacheBean == null) {
            throw new StatusException("400", "验证码错误或已过期，请重试！");
        }

        final String cacheKeyVerifyCodeResource = CacheConstants.CACHE_B_VERIFY_CODE_RESOURCE + cacheBean.getUuid();
        Boolean resource = redisClient.get(cacheKeyVerifyCodeResource, Boolean.class);
        if (resource == null) {
            LOG.warn(String.format("accountValue = %s, uuid = %s, Real uuid = %s", info.getAccountValue(), info.getUuid(), cacheBean.getUuid()));
            throw new StatusException("400", "验证码请求无效，请重试！");
        }

        if (!info.getVerifyCode().equals(cacheBean.getVerifyCode())) {
            redisClient.delete(cacheKeyVerifyCode);
            redisClient.delete(cacheKeyVerifyCodeResource);
            throw new StatusException("400", "验证码输入不正确，请重试！");
        }

        LoginInfo loginInfo = new LoginInfo();
        loginInfo.setRootOrgId(info.getRootOrgId());
        loginInfo.setAccountType(info.getAccountType());
        loginInfo.setAccountValue(info.getAccountValue());
        loginInfo.setPassword(info.getPassword());
        loginInfo.setClientIp(info.getClientIp());
        User user = authService.login(loginInfo, false);

        redisClient.delete(cacheKeyVerifyCode);
        redisClient.delete(cacheKeyVerifyCodeResource);

        //操作日志
        ReportsUtil.report(new OperateReport(user.getRootOrgId(), user.getUserId(), user.getUserId(), null, UserType.STUDENT, OperateContent.STUDENT_LOGIN.getDesc()));

        // 在线学生登录打点
        ReportsUtil.report(new OnlineStudentReport(user.getRootOrgId(), user.getUserId()));

        return user;
    }

    /**
     * 数字公式-验证码的Resource接口
     */
    @Override
    public void verifyCodeResource(String uuid) {
        if (StringUtils.isBlank(uuid) || uuid.length() != 32) {
            LOG.debug("Invalid uuid... " + uuid);
            return;
        }

        final String cacheKeyVerifyCodeResource = CacheConstants.CACHE_B_VERIFY_CODE_RESOURCE + uuid;
        redisClient.set(cacheKeyVerifyCodeResource, true, 60);// N秒
    }

}
